@@ -1,3 +1,22 @@
+//! The autofill decoration automatically fills gaps that appear in the log.
+//!
+//! When a node has a gap in its (local copy of the) log, it must close that gap
+//! before it can apply later rounds' entries to its state. There are two
+//! reasons why a node may observe gaps in its log.
+//!
+//!  - A commit message may not have been received due to a temporary networking
+//!    issue.
+//!  - Multiple appends were running concurrently and one for an earlier round
+//!    fails or is abandoned while while a later one goes through.
+//!
+//! Independent of the cause, the solution is to try to fill the gaps, which is
+//! what this decoration does. It will either succeed or fail with a `Converged`
+//! error.
+//!
+//! It should be noted that autofill uses `Importance::MaintainLeadership` to
+//! fill gaps. As such it will never contend with other nodes that may be in the
+//! process of proposing some other entry for the same round.
+
 use std::collections::BinaryHeap;
 use std::collections::HashSet;
 use std::collections::VecDeque;
@@ -17,21 +36,25 @@ use crate::applicable::ApplicableTo;
 use crate::buffer::Buffer;
 use crate::decoration::Decoration;
 use crate::error::Disoriented;
+use crate::error::ShutDownOr;
 use crate::node::builder::NodeBuilder;
 use crate::node::AbstainOf;
 use crate::node::AppendResultFor;
 use crate::node::CommunicatorOf;
 use crate::node::CoordNumOf;
 use crate::node::EventFor;
+use crate::node::ImplAppendResultFor;
 use crate::node::InvocationOf;
 use crate::node::LogEntryOf;
 use crate::node::NayOf;
 use crate::node::NodeIdOf;
+use crate::node::NodeImpl;
 use crate::node::NodeStatus;
 use crate::node::Participation;
 use crate::node::RoundNumOf;
 use crate::node::SnapshotFor;
 use crate::node::StateOf;
+use crate::node::StaticAppendResultFor;
 use crate::node::YeaOf;
 use crate::retry::DoNotRetry;
 use crate::retry::RetryPolicy;
@@ -40,27 +63,53 @@ use crate::voting::Voter;
 use crate::Node;
 use crate::RoundNum;
 
+/// Autofill configuration.
 pub trait Config {
+    /// The node type that is decorated.
     type Node: Node;
+
+    /// The applicable that is used to fill gaps, usually a no-op.
     type Applicable: ApplicableTo<StateOf<Self::Node>> + 'static;
 
+    /// Type of retry policy to be used.
+    ///
+    /// See [`retry_policy`][Config::retry_policy].
+    type RetryPolicy: RetryPolicy<
+        Invocation = InvocationOf<Self::Node>,
+        Error = AppendError<InvocationOf<Self::Node>>,
+        StaticError = AppendError<InvocationOf<Self::Node>>,
+    >;
+
+    /// Initializes this configuration.
     #[allow(unused_variables)]
     fn init(&mut self, node: &Self::Node) {}
 
+    /// Updates the configuration with the given event.
     #[allow(unused_variables)]
     fn update(&mut self, event: &EventFor<Self::Node>) {}
 
+    /// The number of gap rounds that may be filled concurrently.
     fn batch_size(&self) -> usize;
 
+    /// The amount of time to wait before attempting to fill a gap.
     fn delay(&self) -> Duration;
 
+    /// Creates a new filler value.
     fn new_filler(&self) -> Self::Applicable;
 
-    fn retry_policy(&self) -> Box<dyn RetryPolicy<Invocation = InvocationOf<Self::Node>> + Send> {
-        Box::new(DoNotRetry::new())
-    }
+    /// Creates a retry policy.
+    ///
+    /// Please note that a node that's fallen too far behind may fail to catch
+    /// up using the gap filling approach. This is because other nodes may
+    /// respond with [`Conflict::Converged`][crate::Conflict::Converged] without
+    /// providing a log entry. In these cases, the node must ask another node
+    /// for a recent snapshot and install it. To detect such cases, one may use
+    /// a retry policy that checks for [`AppendError::Converged`] and whether
+    /// the node could be `caught_up`.
+    fn retry_policy(&self) -> Self::RetryPolicy;
 }
 
+/// A static configuration.
 pub struct StaticConfig<N, A> {
     batch_size: usize,
     delay: Duration,
@@ -72,6 +121,7 @@ where
     N: Node,
     A: ApplicableTo<StateOf<N>> + Default + 'static,
 {
+    /// Creates a new static configuration.
     pub fn new(batch_size: usize, delay: Duration) -> Self {
         Self {
             batch_size,
@@ -88,6 +138,7 @@ where
 {
     type Node = N;
     type Applicable = A;
+    type RetryPolicy = DoNotRetry<InvocationOf<N>>;
 
     fn batch_size(&self) -> usize {
         self.batch_size
@@ -100,17 +151,22 @@ where
     fn new_filler(&self) -> Self::Applicable {
         Self::Applicable::default()
     }
+
+    fn retry_policy(&self) -> Self::RetryPolicy {
+        DoNotRetry::new()
+    }
 }
 
+/// Extends `NodeBuilder` to conveniently decorate a node with `Autofill`.
 pub trait AutofillBuilderExt: Sized {
+    /// Node type to be decorated.
     type Node: Node;
+    /// Voter type.
     type Voter: Voter;
-    type Buffer: Buffer<
-        RoundNum = RoundNumOf<Self::Node>,
-        CoordNum = CoordNumOf<Self::Node>,
-        Entry = LogEntryOf<Self::Node>,
-    >;
+    /// Buffer type.
+    type Buffer: Buffer;
 
+    /// Decorates the node with `Autofill` using the given configuration.
     fn fill_gaps<C>(
         self,
         config: C,
@@ -121,7 +177,7 @@ pub trait AutofillBuilderExt: Sized {
 
 impl<N, V, B> AutofillBuilderExt for NodeBuilder<N, V, B>
 where
-    N: Node + 'static,
+    N: NodeImpl + 'static,
     V: Voter<
         State = StateOf<N>,
         RoundNum = RoundNumOf<N>,
@@ -147,6 +203,7 @@ where
     }
 }
 
+/// Autofill decoration.
 #[derive(Debug)]
 pub struct Autofill<N: Node, C> {
     decorated: N,
@@ -283,7 +340,7 @@ where
 
         let append = self
             .decorated
-            .append(
+            .append_static(
                 log_entry,
                 AppendArgs {
                     round: round_num..=round_num,
@@ -300,7 +357,7 @@ where
 
 impl<N, C> Decoration for Autofill<N, C>
 where
-    N: Node,
+    N: NodeImpl,
     C: Config<Node = N> + 'static,
 {
     type Arguments = C;
@@ -443,15 +500,53 @@ where
         self.decorated.read_stale()
     }
 
-    fn append<A: ApplicableTo<StateOf<Self>> + 'static, P: Into<AppendArgs<Self::Invocation>>>(
+    fn append<A, P, R>(
         &self,
         applicable: A,
         args: P,
-    ) -> futures::future::LocalBoxFuture<'static, AppendResultFor<Self, A>> {
+    ) -> futures::future::LocalBoxFuture<'_, AppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
         self.decorated.append(applicable, args)
     }
 
+    fn append_static<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> futures::future::LocalBoxFuture<'static, StaticAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+        R::StaticError: From<ShutDownOr<R::Error>>,
+    {
+        self.decorated.append_static(applicable, args)
+    }
+
     fn shut_down(self) -> Self::Shutdown {
         self.decorated.shut_down()
     }
 }
+
+impl<N, C> NodeImpl for Autofill<N, C>
+where
+    N: NodeImpl,
+    C: Config<Node = N>,
+{
+    fn append_impl<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> LocalBoxFuture<'static, ImplAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
+        self.decorated.append_impl(applicable, args)
+    }
+}
