@@ -1,5 +1,7 @@
+//! Defines the [`Decoration`] trait.
+
 use crate::error::SpawnError;
-use crate::Node;
+use crate::node::NodeImpl;
 
 /// A decoration around a `Node`.
 ///
@@ -7,10 +9,15 @@ use crate::Node;
 /// decoration around a node.
 pub trait Decoration
 where
-    Self: Node,
+    Self: NodeImpl,
 {
+    /// Arguments this decoration requires.
+    ///
+    /// See [`NodeBuilder::decorated_with`][crate::NodeBuilder::decorated_with].
     type Arguments: 'static;
-    type Decorated: Node;
+
+    /// Type of the decorated node.
+    type Decorated: NodeImpl;
 
     /// Wraps this decoration around the given node, using the given arguments.
     fn wrap(decorated: Self::Decorated, arguments: Self::Arguments) -> Result<Self, SpawnError>;
@@ -18,5 +25,6 @@ where
     /// Returns a reference to the decorated node.
     fn peek_into(decorated: &Self) -> &Self::Decorated;
 
+    /// Unwraps this decoration.
     fn unwrap(decorated: Self) -> Self::Decorated;
 }
