@@ -1,3 +1,5 @@
+//! Defines various error types.
+
 use std::convert::Infallible;
 use std::sync::Arc;
 
@@ -11,66 +13,84 @@ use crate::invocation::LogEntryIdOf;
 use crate::invocation::LogEntryOf;
 use crate::invocation::NayOf;
 
-pub type BoxError = Box<dyn std::error::Error + Send + Sync + 'static>;
-
+/// Reason spawning a node failed.
 #[non_exhaustive]
 #[derive(Debug, Error)]
 pub enum SpawnError {
+    /// A decoration's [`wrap` method][crate::decoration::Decoration::wrap]
+    /// failed.
     #[error("a node decoration raised an error")]
-    Decoration(#[source] BoxError),
+    Decoration(#[source] Box<dyn std::error::Error + Send + Sync + 'static>),
 }
 
+/// Reason node snapshot couldn't be created.
 #[derive(Debug, Error)]
 pub enum PrepareSnapshotError {
+    /// Node doesn't have state.
     #[error("node is disoriented")]
     Disoriented,
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
 
+/// Reason node snapshot couldn't be affirmed.
 #[derive(Debug, Error)]
 pub enum AffirmSnapshotError {
+    /// The given snapshot is unknown.
     #[error("unknown snapshot")]
     Unknown,
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
 
+/// Reason node snapshot couldn't be installed.
 #[derive(Debug, Error)]
 pub enum InstallSnapshotError {
+    /// The node's current state is more up-to-date.
     #[error("snapshot is outdated")]
     Outdated,
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
 
+/// Reason node's state couldn't be read.
 #[derive(Debug, Error)]
 pub enum ReadStaleError {
+    /// Node doesn't have state.
     #[error("node is disoriented")]
     Disoriented,
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
 
-/// Preparing a round for proposals failed.
+/// Reason preparing round for proposals failed.
 #[derive(Error)]
 pub enum PrepareError<I: Invocation> {
+    /// Node abstained from voting.
     #[error("promise war deliberately withheld")]
     Abstained(AbstainOf<I>),
 
+    /// Another node es running for leader with a greater coordination number.
     #[error("conflicting promise")]
     Supplanted(CoordNumOf<I>),
 
+    /// Round is already settled.
     #[error("round already converged")]
     Converged(CoordNumOf<I>, Option<(CoordNumOf<I>, Arc<LogEntryOf<I>>)>),
 
+    /// Node is in passive mode.
     #[error("node is passive")]
     Passive,
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
@@ -118,18 +138,23 @@ impl<I: Invocation> From<PrepareError<I>> for AppendError<I> {
 /// A proposal could not be accepted.
 #[derive(Error)]
 pub enum AcceptError<I: Invocation> {
+    /// Another node has become leader with a greater coordination number.
     #[error("conflicting promise")]
     Supplanted(CoordNumOf<I>),
 
+    /// Round is already settled.
     #[error("round already converged")]
     Converged(CoordNumOf<I>, Option<(CoordNumOf<I>, Arc<LogEntryOf<I>>)>),
 
+    /// Node is in passive mode.
     #[error("node is passive")]
     Passive,
 
+    /// Node rejected the proposal.
     #[error("proposal was rejected")]
     Rejected(NayOf<I>),
 
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
@@ -178,6 +203,7 @@ impl<I: Invocation> From<AcceptError<I>> for AppendError<I> {
 #[non_exhaustive]
 #[derive(Error)]
 pub enum CommitError<I: Invocation> {
+    /// Node doesn't have state.
     #[error("node is disoriented")]
     Disoriented,
 
@@ -185,7 +211,7 @@ pub enum CommitError<I: Invocation> {
     #[error("given log entry id is invalid")]
     InvalidEntryId(LogEntryIdOf<I>),
 
-    /// The paxakos node was shut down.
+    /// Node is shut down.
     #[error("node is shut down")]
     ShutDown,
 }
@@ -213,10 +239,12 @@ impl<I: Invocation> From<CommitError<I>> for AppendError<I> {
     }
 }
 
+/// Node doesn't have state.
 #[derive(Clone, Copy, Debug, Error)]
 #[error("node is disoriented")]
 pub struct Disoriented;
 
+/// Node is shut down.
 #[derive(Clone, Copy, Debug, Error)]
 #[error("node is shut down")]
 pub struct ShutDown;
@@ -226,3 +254,45 @@ impl From<Infallible> for ShutDown {
         Self
     }
 }
+
+impl From<ShutDownOr<Infallible>> for ShutDown {
+    fn from(_: ShutDownOr<Infallible>) -> Self {
+        Self
+    }
+}
+
+/// Node is shut down.
+#[derive(Clone, Copy, Debug)]
+pub enum ShutDownOr<E> {
+    /// An event other than 'shut down' occured.
+    Other(E),
+
+    /// Node is shut down.
+    ShutDown,
+}
+
+impl<E> ShutDownOr<E> {
+    /// Applies the mapping function to the other event, if present.
+    pub fn map<T, F: FnOnce(E) -> T>(self, f: F) -> ShutDownOr<T> {
+        match self {
+            ShutDownOr::Other(e) => ShutDownOr::Other(f(e)),
+            ShutDownOr::ShutDown => ShutDownOr::ShutDown,
+        }
+    }
+
+    /// Expect an event other than 'shut down' to have occured.
+    ///
+    /// Panics if a shut down occured.
+    pub fn expect_other(self) -> E {
+        match self {
+            ShutDownOr::Other(e) => e,
+            ShutDownOr::ShutDown => panic!("Node is unexpectedly shut down."),
+        }
+    }
+}
+
+impl<E> From<E> for ShutDownOr<E> {
+    fn from(e: E) -> Self {
+        ShutDownOr::Other(e)
+    }
+}
