@@ -1,3 +1,18 @@
+//! The heartbeats decoration continually appends heartbeat entries to the log.
+//!
+//! Sending heartbeats serves two purposes.
+//!
+//! 1. It establishes a minimum pace at which state is moving forward. This
+//!    enables nodes to detect when they have been disconnected from the
+//!    cluster.
+//! 2. Heartbeats can automatically refresh a [master
+//!    lease][crate::leases::master]. Having a stable master is helpful for any
+//!    services that wish to delegate to it.
+//!
+//! It should be noted that heartbeats uses `Importance::MaintainLeadership` to
+//! append heartbeats. As such it will never contend with other nodes that may
+//! be in the process of proposing some other entry for the same round.
+
 use std::sync::Arc;
 use std::task::Poll;
 use std::time::Duration;
@@ -13,6 +28,7 @@ use crate::applicable::ApplicableTo;
 use crate::buffer::Buffer;
 use crate::decoration::Decoration;
 use crate::error::Disoriented;
+use crate::error::ShutDownOr;
 use crate::leadership::track::MaybeLeadershipAwareNode;
 use crate::node::builder::NodeBuilder;
 use crate::node::AbstainOf;
@@ -20,38 +36,53 @@ use crate::node::AppendResultFor;
 use crate::node::CommunicatorOf;
 use crate::node::CoordNumOf;
 use crate::node::EventFor;
+use crate::node::ImplAppendResultFor;
 use crate::node::InvocationOf;
 use crate::node::LogEntryOf;
 use crate::node::NayOf;
 use crate::node::NodeIdOf;
+use crate::node::NodeImpl;
 use crate::node::NodeStatus;
 use crate::node::Participation;
 use crate::node::RoundNumOf;
 use crate::node::SnapshotFor;
 use crate::node::StateOf;
+use crate::node::StaticAppendResultFor;
 use crate::node::YeaOf;
+use crate::retry::DoNotRetry;
+use crate::retry::RetryPolicy;
 use crate::voting::Voter;
 use crate::Node;
 
+/// Heartbeats configuration.
 pub trait Config {
+    /// The node type that is decorated.
     type Node: Node;
+
+    /// The applicable that is used to fill gaps, usually a no-op.
     type Applicable: ApplicableTo<StateOf<Self::Node>> + 'static;
 
+    /// Initializes this configuration.
     #[allow(unused_variables)]
     fn init(&mut self, node: &Self::Node) {}
 
+    /// Updates the configuration with the given event.
     #[allow(unused_variables)]
     fn update(&mut self, event: &EventFor<Self::Node>) {}
 
+    /// Interval at which leader nodes send heartbeats.
     fn leader_interval(&self) -> Option<Duration> {
         None
     }
 
+    /// Interval at which heartbeats are sent.
     fn interval(&self) -> Option<Duration>;
 
+    /// Creates a new heartbeat value.
     fn new_heartbeat(&self) -> Self::Applicable;
 }
 
+/// A static configuration.
 pub struct StaticConfig<N, A> {
     leader_interval: Option<Duration>,
     interval: Option<Duration>,
@@ -64,6 +95,7 @@ where
     N: Node,
     A: ApplicableTo<StateOf<N>> + Default + 'static,
 {
+    /// Constructs a new configuratin with the given interval.
     pub fn with_interval(interval: Duration) -> Self {
         Self {
             leader_interval: None,
@@ -73,6 +105,7 @@ where
         }
     }
 
+    /// Sets the interval at which leader nodes send heartbeats.
     pub fn when_leading(self, leader_interval: Duration) -> Self {
         Self {
             leader_interval: Some(leader_interval),
@@ -104,15 +137,16 @@ where
     }
 }
 
+/// Extends `NodeBuilder` to conveniently decorate a node with `Heartbeats`.
 pub trait HeartbeatsBuilderExt<I = ()> {
+    /// Node type to be decorated.
     type Node: MaybeLeadershipAwareNode<I> + 'static;
+    /// Voter type.
     type Voter: Voter;
-    type Buffer: Buffer<
-        RoundNum = RoundNumOf<Self::Node>,
-        CoordNum = CoordNumOf<Self::Node>,
-        Entry = LogEntryOf<Self::Node>,
-    >;
+    /// Buffer type.
+    type Buffer: Buffer;
 
+    /// Decorates the node with `Heartbeats` using the given configuration.
     #[allow(clippy::type_complexity)]
     fn send_heartbeats<C>(
         self,
@@ -124,7 +158,7 @@ pub trait HeartbeatsBuilderExt<I = ()> {
 
 impl<N, V, B, I> HeartbeatsBuilderExt<I> for NodeBuilder<N, V, B>
 where
-    N: MaybeLeadershipAwareNode<I> + 'static,
+    N: NodeImpl + MaybeLeadershipAwareNode<I> + 'static,
     V: Voter<
         State = StateOf<N>,
         RoundNum = RoundNumOf<N>,
@@ -151,6 +185,7 @@ where
     }
 }
 
+/// Heartbeats decoration.
 #[derive(Debug)]
 pub struct Heartbeats<N, C, I = ()>
 where
@@ -205,9 +240,10 @@ where
     fn send_heartbeat(&mut self) {
         let append = self
             .decorated
-            .append(
+            .append_static(
                 self.config.new_heartbeat(),
                 AppendArgs {
+                    retry_policy: DoNotRetry::new(),
                     importance: Importance::MaintainLeadership(Peeryness::Peery),
                     ..Default::default()
                 },
@@ -221,7 +257,7 @@ where
 
 impl<N, C, I> Decoration for Heartbeats<N, C, I>
 where
-    N: MaybeLeadershipAwareNode<I> + 'static,
+    N: NodeImpl + MaybeLeadershipAwareNode<I> + 'static,
     C: Config<Node = N> + 'static,
 {
     type Arguments = C;
@@ -348,15 +384,53 @@ where
         self.decorated.read_stale()
     }
 
-    fn append<A: ApplicableTo<StateOf<Self>> + 'static, P: Into<AppendArgs<Self::Invocation>>>(
+    fn append<A, P, R>(
         &self,
         applicable: A,
         args: P,
-    ) -> futures::future::LocalBoxFuture<'static, AppendResultFor<Self, A>> {
+    ) -> futures::future::LocalBoxFuture<'_, AppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
         self.decorated.append(applicable, args)
     }
 
+    fn append_static<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> futures::future::LocalBoxFuture<'static, StaticAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+        R::StaticError: From<ShutDownOr<R::Error>>,
+    {
+        self.decorated.append_static(applicable, args)
+    }
+
     fn shut_down(self) -> Self::Shutdown {
         self.decorated.shut_down()
     }
 }
+
+impl<N, C, I> NodeImpl for Heartbeats<N, C, I>
+where
+    N: NodeImpl + MaybeLeadershipAwareNode<I> + 'static,
+    C: Config<Node = N>,
+{
+    fn append_impl<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> LocalBoxFuture<'static, ImplAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
+        self.decorated.append_impl(applicable, args)
+    }
+}
