@@ -10,22 +10,27 @@ use crate::applicable::ApplicableTo;
 use crate::buffer::Buffer;
 use crate::decoration::Decoration;
 use crate::error::Disoriented;
+use crate::error::ShutDownOr;
 use crate::node::builder::NodeBuilder;
 use crate::node::AbstainOf;
 use crate::node::AppendResultFor;
 use crate::node::CommunicatorOf;
 use crate::node::CoordNumOf;
 use crate::node::EventFor;
+use crate::node::ImplAppendResultFor;
 use crate::node::InvocationOf;
 use crate::node::LogEntryOf;
 use crate::node::NayOf;
 use crate::node::NodeIdOf;
+use crate::node::NodeImpl;
 use crate::node::NodeStatus;
 use crate::node::Participation;
 use crate::node::RoundNumOf;
 use crate::node::SnapshotFor;
 use crate::node::StateOf;
+use crate::node::StaticAppendResultFor;
 use crate::node::YeaOf;
+use crate::retry::RetryPolicy;
 use crate::voting::Voter;
 use crate::Node;
 use crate::RoundNum;
@@ -60,7 +65,7 @@ pub struct EnsureLeadershipBuilder<N, P> {
 
 impl<N, V, B> EnsureLeadershipBuilderExt for NodeBuilder<N, V, B>
 where
-    N: Node + 'static,
+    N: NodeImpl + 'static,
     V: Voter<
         State = StateOf<N>,
         RoundNum = RoundNumOf<N>,
@@ -194,7 +199,7 @@ where
 
         let append = self
             .decorated
-            .append(log_entry, ())
+            .append_static(log_entry, ())
             .map(|_| ())
             .boxed_local();
 
@@ -204,7 +209,7 @@ where
 
 impl<N, P> Decoration for EnsureLeadership<N, P>
 where
-    N: Node + 'static,
+    N: NodeImpl + 'static,
     P: Fn() -> LogEntryOf<N> + 'static,
 {
     type Arguments = EnsureLeadershipArgs<N, P>;
@@ -324,15 +329,53 @@ where
         self.decorated.read_stale()
     }
 
-    fn append<A: ApplicableTo<StateOf<Self>> + 'static, P: Into<AppendArgs<Self::Invocation>>>(
+    fn append<A, P, R>(
         &self,
         applicable: A,
         args: P,
-    ) -> futures::future::LocalBoxFuture<'static, AppendResultFor<Self, A>> {
+    ) -> futures::future::LocalBoxFuture<'_, AppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
         self.decorated.append(applicable, args)
     }
 
+    fn append_static<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> futures::future::LocalBoxFuture<'static, StaticAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+        R::StaticError: From<ShutDownOr<R::Error>>,
+    {
+        self.decorated.append_static(applicable, args)
+    }
+
     fn shut_down(self) -> Self::Shutdown {
         self.decorated.shut_down()
     }
 }
+
+impl<N, F> NodeImpl for EnsureLeadership<N, F>
+where
+    N: NodeImpl + 'static,
+    F: Fn() -> LogEntryOf<N> + 'static,
+{
+    fn append_impl<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> LocalBoxFuture<'static, ImplAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
+        self.decorated.append_impl(applicable, args)
+    }
+}
