@@ -15,6 +15,7 @@ use crate::buffer::Buffer;
 use crate::communicator::Communicator;
 use crate::decoration::Decoration;
 use crate::error::Disoriented;
+use crate::error::ShutDownOr;
 use crate::event::DirectiveKind;
 use crate::invocation;
 use crate::invocation::Invocation;
@@ -25,11 +26,13 @@ use crate::node::CommunicatorOf;
 use crate::node::CoordNumOf;
 use crate::node::Core;
 use crate::node::EventFor;
+use crate::node::ImplAppendResultFor;
 use crate::node::InvocationOf;
 use crate::node::LogEntryOf;
 use crate::node::NayOf;
 use crate::node::Node;
 use crate::node::NodeIdOf;
+use crate::node::NodeImpl;
 use crate::node::NodeInfo;
 use crate::node::NodeOf;
 use crate::node::NodeStatus;
@@ -37,7 +40,9 @@ use crate::node::Participation;
 use crate::node::RoundNumOf;
 use crate::node::SnapshotFor;
 use crate::node::StateOf;
+use crate::node::StaticAppendResultFor;
 use crate::node::YeaOf;
+use crate::retry::RetryPolicy;
 use crate::voting::Voter;
 use crate::RoundNum;
 use crate::Shell;
@@ -48,10 +53,10 @@ pub trait LeadershipAwareNode<I>: Node {
     /// Leadership as assumed by this node.
     ///
     /// Note: This information is unreliable. A node will assume itself or
-    /// another node to be leader when that node "owns" the highest coordination
-    /// number observed for that round. That means neither that the node managed
-    /// to achieve a quorum with that number, nor that no other node
-    /// achieved a quorum with an even higher number.
+    /// another node to be leader when that node "owns" the greatest
+    /// coordination number observed for that round. That means neither that
+    /// the node managed to achieve a quorum with that number, nor that no other
+    /// node achieved a quorum with an even higher number.
     fn lax_leadership(&self) -> &[LeadershipFor<Self>];
 
     /// Leadership as assumed by this node.
@@ -84,7 +89,7 @@ pub trait TrackLeadershipBuilderExt {
 
 impl<N, V, B> TrackLeadershipBuilderExt for NodeBuilder<N, V, B>
 where
-    N: Node + 'static,
+    N: NodeImpl + 'static,
     V: Voter<
         State = StateOf<N>,
         RoundNum = RoundNumOf<N>,
@@ -240,7 +245,7 @@ impl<N: Node> Inferrer<N> {
 
 impl<N> Decoration for TrackLeadership<N>
 where
-    N: Node + 'static,
+    N: NodeImpl + 'static,
 {
     type Arguments = ();
 
@@ -372,19 +377,56 @@ where
         self.decorated.read_stale()
     }
 
-    fn append<A: ApplicableTo<StateOf<Self>> + 'static, P: Into<AppendArgs<Self::Invocation>>>(
+    fn append<A, P, R>(
         &self,
         applicable: A,
         args: P,
-    ) -> futures::future::LocalBoxFuture<'static, AppendResultFor<Self, A>> {
+    ) -> futures::future::LocalBoxFuture<'_, AppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
         self.decorated.append(applicable, args)
     }
 
+    fn append_static<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> futures::future::LocalBoxFuture<'static, StaticAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+        R::StaticError: From<ShutDownOr<R::Error>>,
+    {
+        self.decorated.append_static(applicable, args)
+    }
+
     fn shut_down(self) -> Self::Shutdown {
         self.decorated.shut_down()
     }
 }
 
+impl<N> NodeImpl for TrackLeadership<N>
+where
+    N: NodeImpl,
+{
+    fn append_impl<A, P, R>(
+        &self,
+        applicable: A,
+        args: P,
+    ) -> LocalBoxFuture<'static, ImplAppendResultFor<Self, A, R>>
+    where
+        A: ApplicableTo<StateOf<Self>> + 'static,
+        P: Into<AppendArgs<Self::Invocation, R>>,
+        R: RetryPolicy<Invocation = Self::Invocation>,
+    {
+        self.decorated.append_impl(applicable, args)
+    }
+}
+
 impl<D, I> LeadershipAwareNode<(I,)> for D
 where
     D: Decoration
@@ -405,7 +447,7 @@ where
 
 impl<N, I> LeadershipAwareNode<(I,)> for Shell<N>
 where
-    N: Node + LeadershipAwareNode<I>,
+    N: NodeImpl + LeadershipAwareNode<I>,
 {
     fn lax_leadership(&self) -> &[LeadershipFor<N>] {
         self.wrapped.lax_leadership()
