@@ -68,7 +68,7 @@
 //!
 //!     type LogEntry = CalcOp;
 //!     type Outcome = f64;
-//!     type Event = f64;
+//!     type Effect = f64;
 //!
 //! #   type Node = ();
 //! #   
@@ -80,7 +80,7 @@
 //!         &mut self,
 //!         log_entry: &Self::LogEntry,
 //!         _context: &mut (),
-//!     ) -> (Self::Outcome, Self::Event) {
+//!     ) -> (Self::Outcome, Self::Effect) {
 //!         if self.applied.insert(log_entry.id()) {
 //!             match log_entry {
 //!                 CalcOp::Add(v, _) => {
@@ -276,7 +276,7 @@
 //!
 //!    2. Conflict `(CoordNum, Option<(CoordNum, LogEntry)>)`
 //!
-//!       A rejection is sent with the highest observed coordination number so
+//!       A rejection is sent with the greatest observed coordination number so
 //!       far. For the special case that the round has already converged and the
 //!       node still has it available, it will send it along as well.
 //!
@@ -296,7 +296,7 @@
 //!
 //!    The only restriction is that it must respect the promises it has
 //!    received. If multiple promises contain a triple with the same round
-//!    number, the one with the highest coordination number wins. (Triples with
+//!    number, the one with the greatest coordination number wins. (Triples with
 //!    the same round and coordination number will have the same log entry as
 //!    well.)
 //!
@@ -380,6 +380,7 @@
 // Lint configuration
 #![warn(rust_2018_idioms)]
 #![warn(clippy::wildcard_imports)]
+#![warn(missing_docs)]
 
 pub mod append;
 pub mod applicable;
@@ -505,10 +506,21 @@ pub trait Identifier: 'static + Copy + Debug + Eq + Hash + Send + Sync + Unpin {
 impl<T: 'static + Copy + Debug + Eq + Hash + Ord + Send + Sync + Unpin> Identifier for T {}
 
 /// A condition for a [Promise].
+///
+/// When a node _promises_, i.e. votes for a candidate, it might require the
+/// candidate to honor commitments the node had previously made. This struct
+/// represents such a requirement.
+///
+/// A `Condition` is a triple of round number, coordination number and log
+/// entry. At [the protocol level](crate#protocol), this triple corresponds to a
+/// proposal that the node previously accepted.
 #[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 pub struct Condition<R, C, E> {
+    /// The condition's round number.
     pub round_num: R,
+    /// The condition's coordination number.
     pub coord_num: C,
+    /// The condition's log entry.
     pub log_entry: Arc<E>,
 }
 
@@ -640,11 +652,22 @@ impl<R: RoundNum, C: CoordNum, E> Promise<R, C, E> {
 /// Please refer to the [description of the protocol](crate#protocol).
 #[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 pub enum Conflict<C, E> {
+    /// Another node had itself elected leader.
     Supplanted {
+        /// The coordination number the other node used to become leader.
         coord_num: C,
     },
+    /// The given round has already converged.
     Converged {
+        /// The greatest coordination number the node that sent the `Converged`
+        /// message had observed.
         coord_num: C,
+        /// The coordination number that was used to settle the round and the
+        /// log entry which was committed.
+        ///
+        /// May be `None` if the node that sent the `Converged` message no
+        /// longer had this information in its [applied entry
+        /// buffer][crate::buffer::Buffer].
         log_entry: Option<(C, Arc<E>)>,
     },
 }
