@@ -10,6 +10,7 @@ use crate::state::State;
 use crate::CoordNum;
 use crate::RoundNum;
 
+/// A snapshot of a node's state.
 #[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 #[serde(bound(
     serialize = "S: serde::Serialize",
@@ -34,8 +35,8 @@ pub enum VersionedSnapshot<S: State, R: RoundNum, C: CoordNum> {
 pub struct SnapshotV1<S: State, R: RoundNum, C: CoordNum> {
     round: R,
     state: Arc<S>,
-    highest_observed_round_num: Option<R>,
-    highest_observed_coord_num: C,
+    greatest_observed_round_num: Option<R>,
+    greatest_observed_coord_num: C,
     participation: Participation<R, C>,
     promises: BTreeMap<R, C>,
     accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
@@ -71,8 +72,8 @@ impl<R: RoundNum, C: CoordNum> Participation<R, C> {
 pub(crate) struct DeconstructedSnapshot<S: State, R: RoundNum, C: CoordNum> {
     pub(crate) round: R,
     pub(crate) state: Arc<S>,
-    pub(crate) highest_observed_round_num: Option<R>,
-    pub(crate) highest_observed_coord_num: C,
+    pub(crate) greatest_observed_round_num: Option<R>,
+    pub(crate) greatest_observed_coord_num: C,
     pub(crate) participation: Participation<R, C>,
     pub(crate) promises: BTreeMap<R, C>,
     pub(crate) accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
@@ -82,8 +83,8 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
     pub(crate) fn new(
         round: R,
         state: Arc<S>,
-        highest_observed_round_num: Option<R>,
-        highest_observed_coord_num: C,
+        greatest_observed_round_num: Option<R>,
+        greatest_observed_coord_num: C,
         participation: Participation<R, C>,
         promises: BTreeMap<R, C>,
         accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
@@ -91,14 +92,15 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
         Self(VersionedSnapshot::V1(SnapshotV1 {
             round,
             state,
-            highest_observed_round_num,
-            highest_observed_coord_num,
+            greatest_observed_round_num,
+            greatest_observed_coord_num,
             participation,
             promises,
             accepted_entries,
         }))
     }
 
+    /// Creates an "initial" snapshot, i.e. a snapshot for round `0`.
     pub fn initial(state: impl Into<Arc<S>>) -> Self {
         let mut promises = BTreeMap::new();
         promises.insert(Zero::zero(), One::one());
@@ -114,12 +116,14 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
         )
     }
 
+    /// The round the node was in when the snapshot was taken.
     pub fn round(&self) -> R {
         match &self.0 {
             VersionedSnapshot::V1(s) => s.round,
         }
     }
 
+    /// The state the node was in when the snapshot was taken.
     pub fn state(&self) -> &Arc<S> {
         match &self.0 {
             VersionedSnapshot::V1(s) => &s.state,
@@ -131,8 +135,8 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
             VersionedSnapshot::V1(s) => DeconstructedSnapshot {
                 round: s.round,
                 state: s.state,
-                highest_observed_round_num: s.highest_observed_round_num,
-                highest_observed_coord_num: s.highest_observed_coord_num,
+                greatest_observed_round_num: s.greatest_observed_round_num,
+                greatest_observed_coord_num: s.greatest_observed_coord_num,
                 participation: s.participation,
                 promises: s.promises,
                 accepted_entries: s.accepted_entries,
